#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Created by J.B.R. OONK  (ASTRON/LEIDEN UNIV. 2015)
Edited by Marco Iacobelli, 7 March 2021
Syntax at the terminal: python3 untarMS.py <in_dir> --<html_check> --<debug>


This is a pipeline to untar and check completeness of the retrieved MS files. Inputs needed: the working directory and the html.txt file.

Versions
--------
v0.1  MI Initial version
"""

import argparse,glob,os,sys


def noemptyline(in_path,in_txt_file):
	"""
	Check and remove empty lines by opening and replacing the .txt file 
		
	Parameters
	----------
	in_txt_file : str
		The path collecting the txt file to be checked
	
	Returns
	-------
	result : file
		The empty lines removed txt file.
	"""
	in_file = in_path+in_txt_file
	tmp_file = in_path+'/tmp.txt'
	with open(in_file, 'r') as r, open(tmp_file, 'w') as o:
		for line in r:
			if not line.isspace(): 
				o.write(line)
	
	o.close()
	os.system('rm -f '+str(in_file)+'')
	os.system('mv '+str(tmp_file)+' '+str(in_file)+'')
	return  in_file


def main(in_ms_dir,ms_check,Debug):
	"""
	Get the list of files to be extracted.
	
	Parameters
	----------
	in_ms_dir : str
		The path collecting the (tar) files
	ms_check : bool,optional
		Check the content of tar files in the input directory vs the expected list of MS files in the html file
	in_debug: bool, optional
		Print auxiliary info
	
	Returns
	-------
	result : array
		Formatted list of filenames paths.
	"""
	#
	# check inputs
	#
	path = in_ms_dir #path = "./"  # FILE DIRECTORY
	filelist = sorted(glob.glob(path+'*.tar'))
	if len(filelist) == 0:
		print("\n Invalid input path: please select a path collecting the tar MS files! \n")
		sys.exit(1)
	
	if ms_check:
		if os.path.isfile(path+'/html.txt'):
			# file exists
			f = open(path+'/html.txt')
			content = f.readlines()
			f.close()
		else:
			print("\n No html.txt file found in the input path! No completeness check can be done! \n")
			
	if Debug: print('LIST:', filelist)
	#
	#FILE STRING SEPARATORS
	sp1d='%'
	sp2d='2F'
	extn='.MS'
	extt='.tar'
	
	#LOOP
	print('#####  STARTING THE LOOP  #####')
	for infile_orig in filelist:
	
		#GET FILE
		infiletar  = os.path.basename(infile_orig)
		infile     = infiletar
		print('doing file: ', infile)
		
		nrspl = len(infile.split(sp1d))
		spl1=infile.split(sp1d)[nrspl-1]
		#spl1=infile.split(sp1d)[11]
		spl2=spl1.split(sp2d)[1]
		spl3=spl2.split(extn)[0]
		newname = spl3+extn+extt
		  
		inFILE = path+infile
		newFILE = path+newname
		# SPECIFY FILE MV COMMAND
		command='mv ' + inFILE + ' ' +newFILE
		print(command)
		
		# CARRY OUT FILENAME CHANGE !!!
		# - COMMENT FOR TESTING OUTPUT
		# - UNCOMMENT TO PERFORM FILE MV COMMAND
		os.system(command)
		os.system('tar -xvf '+newFILE)
		os.system('rm -r '+newFILE)
		
		print('finished rename / untar / remove of: ', newFILE)
	
	print('\n Total number of files is %d \n' % len(filelist))
	if ms_check: 
		print(' Found %d files to be retrieved and %d on disk: \n' % (len(content),len(filelist)))
		if len(content) == len(filelist):
			print(' No further action needed . . .\n')
		else:
			# Opening a file for appending
			html_missed = open("html_missed.txt", "a")
			print(' Checking for missing files . . .\n')
			# find missing files, get SURLs from the HTML.txt file and write to another html_missed.txt file
			msfilelist = sorted(glob.glob(path+'*.MS'))
			if Debug: print(msfilelist)
			aux = 0
			for i in range(len(content)):
				ms_name = '_'.join((content[i].split("/")[-1]).split("_")[:3])+'_uv.MS'
				if path+ms_name not in msfilelist:
					if Debug: print('WARNING: file %s not found !' % ms_name)
					# append SURL to the html_missed.txt file
					print(content[i]+'\n', file=html_missed)
					aux = aux +1
			#
			html_missed.close()
			if aux == 0:
				os.system('rm -rf '+str(html_missed)+'')
			else:
				# try to retrieve via wget the missing files
				print('\n Found %d missed files; attempting to download these . . . \n' % aux)
				noemptyline(path,'html_missed.txt')
				if Debug:
					print('Running wget -i html_missed.txt')
				else:
					os.system('wget -i html_missed.txt')



if __name__ == "__main__":
	descriptiontext = " Script to untar and check completeness of the retrieved MS files. Inputs needed: the working directory and the html.txt file.\n"
	
	parser = argparse.ArgumentParser(description=descriptiontext, formatter_class=argparse.RawTextHelpFormatter)
	parser.add_argument('in_dir', help='The input directory collecting input MS files', type=str)
	parser.add_argument('--html_check', help='Check the completeness of the downloaded MS tar files; if True a html.txt file is expected in the input directory (default False)', action='store_true', default=False)
	parser.add_argument('--debug',help='Run in debug mode', action='store_true', default=False)
	args = parser.parse_args()
	
	inDIR = args.in_dir
	msCHECK = args.html_check
	Debug = args.debug
	main(inDIR,msCHECK,Debug)
